# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import print_function

try:
    from gi import require_version
except ImportError:
    print("ERROR: pygobject is not installed.")
    raise
try:
    require_version('PangoCairo', '1.0')
except ValueError:
    print("ERROR: PangoCairo development files are not installed.")
    raise
from gi.repository import Pango
from gi.repository import PangoCairo


class printing():

    def begin_print(self, operation, print_ctx, print_data):
        page_dpi_y = print_ctx.get_dpi_y()
        page_dpi_x = print_ctx.get_dpi_x()
        print('page_dpi_x:', page_dpi_x, 'page_dpi_y:', page_dpi_y)

        page_y_px = print_ctx.get_height(
            ) - self.HEADER_HEIGHT - self.HEADER_GAP
        page_x_px = print_ctx.get_width()
        print('page_x_px:', page_x_px, 'page_y_px:', page_y_px)

        page_dpi_y = print_ctx.get_dpi_y()
        page_dpi_x = print_ctx.get_dpi_x()
        print('page_dpi_x:', page_dpi_x, 'page_dpi_y:', page_dpi_y)

        page_x_mm = 25.4 * page_x_px / page_dpi_x
        page_y_mm = 25.4 * page_y_px / page_dpi_y
        print('page_x_mm:', page_x_mm, 'page_y_mm:', page_y_mm)

        label_x_mm, label_y_mm = self.get_label_size()
        print('label_x_mm:', label_x_mm, 'label_y_mm:', label_y_mm)

        labels_x = int(page_x_mm / label_x_mm)
        labels_y = int(page_y_mm / label_y_mm)
        print_data['labels_page'] = labels_x * labels_y
        print(
            'labels_x:', labels_x,
            'labels_y:', labels_y,
            'labels_page:', print_data['labels_page']
            )
        print_data['n_labels'] = len(
            print_data['labels']
            ) * print_data['copies']
        print_data['num_pages'] = (
            print_data['n_labels'] - 1
            ) / print_data['labels_page'] + 1
        print('num_pages:', print_data['num_pages'])
        operation.set_n_pages(print_data['num_pages'])

    def draw_page(self, operation, print_ctx, page_num, print_data):
        dpi_x = print_ctx.get_dpi_x()
        print(
            "draw_page dpi_x:", dpi_x,
            operation.get_print_settings().get_resolution_x()
            # cairo_surface_get_fallback_resolution
            )
        cr = print_ctx.get_cairo_context()
        width = print_ctx.get_width()
        cr.rectangle(0, 0, width, self.HEADER_HEIGHT)
        cr.set_source_rgb(0.8, 0.8, 0.8)
        cr.fill_preserve()
        cr.set_source_rgb(0, 0, 0)
        cr.set_line_width(1)
        cr.stroke()
        layout = print_ctx.create_pango_layout()
        desc = Pango.FontDescription('sans 14')
        layout.set_font_description(desc)
        # layout.set_text(print_data['filename'], -1)
        (text_width, text_height) = layout.get_pixel_size()
        if text_width > width:
            layout.set_width(width)
            layout.set_ellipsize(Pango.EllipsizeMode.START)
            (text_width, text_height) = layout.get_pixel_size()
        cr.move_to((width - text_width) / 2,
                   (self.HEADER_HEIGHT - text_height) / 2)
        PangoCairo.show_layout(cr, layout)
        page_str = "%d/%d" % (page_num + 1, print_data['num_pages'])
        layout.set_text(page_str, -1)
        layout.set_width(-1)
        (text_width, text_height) = layout.get_pixel_size()
        cr.move_to(width - text_width - 4,
                   (self.HEADER_HEIGHT - text_height) / 2)
        PangoCairo.show_layout(cr, layout)
        layout = print_ctx.create_pango_layout()
        # desc = Pango.FontDescription('monospace')
        # desc.set_size(print_data['font_size'] * Pango.SCALE)
        # layout.set_font_description(desc)
        cr.move_to(0, self.HEADER_HEIGHT + self.HEADER_GAP)
        for label in print_data['labels']:
            label_file = self.get_label_file(label)
            print('label_file:', label_file)
            self.draw_label_file(label_file, cr, dpi_x)
            return
        # lines_pp = int(print_data['lines_per_page'])
        # num_lines = print_data['num_lines']
        # data_lines = print_data['lines']
        # font_size = print_data['font_size']
        # line = page_num * lines_pp
        # for i in range(lines_pp):
        #     if line >= num_lines:
        #         break
        #     layout.set_text(data_lines[line], -1)
        #     PangoCairo.show_layout(cr, layout)
        #     cr.rel_move_to(0, font_size)
        #     line += 1

    def end_print(self, operation, print_ctx, print_data):
        pass
